# This file contains a list of definitions for various Fischer groups

# ConjClassToFischer := function(G,Din)
# 	local Gperm, actHom;
# 	Display("Computing permutation rep of G on D... ");
# 	actHom := ActionHomomorphism(G, Din);
# 	Gperm := Image(actHom);
# 	return rec(G := Gperm, d := Image(actHom, Din[1]), D := MovedPoints(Gperm));
# end;;
# 
# UnitaryFischer := function(n)
# 	local G, D;
# 	G:=SU(n,2);
# 	D := Transvections(G);;
# 	#Print("|D| = ", Size(D), "\n");
# 	Assert(0, Size(D) = 1/6 * (4^n + (-2)^n - 2));
# 	return ConjClassToFischer(G, D);
# end;;


# TODO:
# Put 3-transposition groups into a record, with entries for
# * the original group (do we really need this?)
# * the original transvection (do we really need this?)
# * Gperm, perm rep on the orbit of the group on 3-transposition,
# * image of the original transvection in Gperm
# * ...
# 

# One with the original group / transvection


if n > 0 and n < 20 then
	#n:=5;
	Print("Computing SU(", n, ",2)\n");
	G:=SU(n,2);

	Print("Computing transvections\n");
	
	# Compute all transvections = points
	D := Transvections(G);;
	Print("|D| = ", Size(D), "\n");
	d1 := D[1];
	Assert(0, Size(D) = 1/6 * (4^n + (-2)^n - 2));

elif n in [ 1903..1920 ] then
	#
	# Determine 3^n:W(A_n)
	#
	n := n - 1900;
	Print("Computing 3^", n, ":W(A_", n, ")\n");

	W:=SymmetricGroup(n+1);
	M:=Group(List(GeneratorsOfGroup(W),g->PermutationMat(g,n+1,GF(3)))); 
	
	V := FieldOfMatrixGroup( M ) ^ DimensionOfMatrixGroup( M );
	G := Action( M, V );
	v:=Basis( V )[1] - Basis( V )[2];
	g:=Permutation( v, V, \+ );;
	B:=ClosureGroup(G,g);
	Order(B.2);
	Size(ConjugacyClass(B,B.2));
	
	#Assert(0, XXXX = Size(H));
	
	G := B;
	d1 := B.2;
	Assert(0, Size(G) = 3^n*Factorial(n+1));

	Print("Computing transvections\n");
	D := Elements(ConjugacyClass(G, d1));;
	Print("|D| = ", Size(D), "\n");

	# 3^3 : W(A_3)
	# |D| = 18
	# dim(A/V) = 14
	# dim(V) = 4
	# dim(L) = 15   (FIXME:   "real dim(L) = 6" or 10 in some runs, hu?
	# 
	# 3^4 : W(A_4)
	# |D| = 30
	# dim(A/V) = 24
	# dim(V) = 6
	# dim(L) = 24
	# 
	# 3^5 : W(A_5)
	# |D| = 45
	# dim(A/V) = 34
	# dim(V) = 11
	# dim(L) = 35
	# 
	# 3^6 : W(A_6)
	# |D| = 63        = 3 * (1+2+3+4+5+6)
	# dim(A/V) = 48
	# dim(V) = 15
	# dim(L) = 48
	# 
	# 3^7 : W(A_7)
	# |D| = 84        = 3 * (1+2+3+4+5+6+7)
	# dim(A/V) = 62
	# dim(V) = 22
	# dim(L) = 63  (FIXME:  real dim(L) = 69 in one run ????)

elif n in [ 2004..2020 ] then
	#
	# Determine 3^n:W(D_n)
	#
	n := n - 2000;
	Print("Computing 3^", n, ":W(D_", n, ")\n");

	# Compute a generating set for W(D_n)
	gens := [];
	for i in [1..n] do
		mat := IdentityMat(n);
		mat[i][i] := -1;
		if i=1 then
			mat[3][i] := 1;
		elif i=2 then
			mat[3][i] := 1;
		elif i=3 then
			mat[1][i] := 1;
		fi;
		if i>=3 then
			mat[i-1][i] := 1;
			if i+1 <= n then
				mat[i+1][i] := 1;
			fi;
		fi;
		Add(gens, mat);
	od;

	H:=Group(gens * One(GF(3)));
	Assert(0, Size(H) = 2^(n-1)*Factorial(n));
	
	#Assert(0, XXXX = Size(H));
	
	G := SemidirectProduct(H, GF(3)^n);
	d1 := G.1;

	Print("Computing transvections\n");
	D := Elements(ConjugacyClass(G, d1));;
	Print("|D| = ", Size(D), "\n");

#gap> List([d4gens, d5gens, d6gens, d7gens], g -> Size(Group(g)));
#[ 192, 1920, 23040, 322560 ]

	# 3^4 : W(D_4)
	# |D| = 36
	# dim(A/V) = 26
	# dim(V) = 10
	# dim(L) = 28
	# 
	# 3^5 : W(D_5)
	# |D| = 60
	# dim(A/V) = 44
	# dim(V) = 16
	# dim(L) = 45
	# 
	# 3^6 : W(D_6)
	# |D| = 90
	# dim(A/V) = 64
	# dim(V) = 26
	# dim(L) = 66
	# 
	# 3^7 : W(D_7)
	# |D| = 126
	# dim(A/V) = 90
	# dim(V) = 36
	# dim(L) = 91

elif n in [ 2106..2108 ] then

	#
	# Determine 3^8:W(E_8)
	#
	n := n - 2100;
	Print("Computing 3^", n, ":W(E_", n, ")\n");

	E8gens := [ [ [ -1, 0, 0, 0, 0, 0, 0, 0 ], [ 0, 1, 0, 0, 0, 0, 0, 0 ], 
      [ 1, 0, 1, 0, 0, 0, 0, 0 ], [ 0, 0, 0, 1, 0, 0, 0, 0 ], 
      [ 0, 0, 0, 0, 1, 0, 0, 0 ], [ 0, 0, 0, 0, 0, 1, 0, 0 ], 
      [ 0, 0, 0, 0, 0, 0, 1, 0 ], [ 0, 0, 0, 0, 0, 0, 0, 1 ] ], 
  [ [ 1, 0, 0, 0, 0, 0, 0, 0 ], [ 0, -1, 0, 0, 0, 0, 0, 0 ], 
      [ 0, 0, 1, 0, 0, 0, 0, 0 ], [ 0, 1, 0, 1, 0, 0, 0, 0 ], 
      [ 0, 0, 0, 0, 1, 0, 0, 0 ], [ 0, 0, 0, 0, 0, 1, 0, 0 ], 
      [ 0, 0, 0, 0, 0, 0, 1, 0 ], [ 0, 0, 0, 0, 0, 0, 0, 1 ] ], 
  [ [ 1, 0, 1, 0, 0, 0, 0, 0 ], [ 0, 1, 0, 0, 0, 0, 0, 0 ], 
      [ 0, 0, -1, 0, 0, 0, 0, 0 ], [ 0, 0, 1, 1, 0, 0, 0, 0 ], 
      [ 0, 0, 0, 0, 1, 0, 0, 0 ], [ 0, 0, 0, 0, 0, 1, 0, 0 ], 
      [ 0, 0, 0, 0, 0, 0, 1, 0 ], [ 0, 0, 0, 0, 0, 0, 0, 1 ] ], 
  [ [ 1, 0, 0, 0, 0, 0, 0, 0 ], [ 0, 1, 0, 1, 0, 0, 0, 0 ], 
      [ 0, 0, 1, 1, 0, 0, 0, 0 ], [ 0, 0, 0, -1, 0, 0, 0, 0 ], 
      [ 0, 0, 0, 1, 1, 0, 0, 0 ], [ 0, 0, 0, 0, 0, 1, 0, 0 ], 
      [ 0, 0, 0, 0, 0, 0, 1, 0 ], [ 0, 0, 0, 0, 0, 0, 0, 1 ] ], 
  [ [ 1, 0, 0, 0, 0, 0, 0, 0 ], [ 0, 1, 0, 0, 0, 0, 0, 0 ], 
      [ 0, 0, 1, 0, 0, 0, 0, 0 ], [ 0, 0, 0, 1, 1, 0, 0, 0 ], 
      [ 0, 0, 0, 0, -1, 0, 0, 0 ], [ 0, 0, 0, 0, 1, 1, 0, 0 ], 
      [ 0, 0, 0, 0, 0, 0, 1, 0 ], [ 0, 0, 0, 0, 0, 0, 0, 1 ] ], 
  [ [ 1, 0, 0, 0, 0, 0, 0, 0 ], [ 0, 1, 0, 0, 0, 0, 0, 0 ], 
      [ 0, 0, 1, 0, 0, 0, 0, 0 ], [ 0, 0, 0, 1, 0, 0, 0, 0 ], 
      [ 0, 0, 0, 0, 1, 1, 0, 0 ], [ 0, 0, 0, 0, 0, -1, 0, 0 ], 
      [ 0, 0, 0, 0, 0, 1, 1, 0 ], [ 0, 0, 0, 0, 0, 0, 0, 1 ] ], 
  [ [ 1, 0, 0, 0, 0, 0, 0, 0 ], [ 0, 1, 0, 0, 0, 0, 0, 0 ], 
      [ 0, 0, 1, 0, 0, 0, 0, 0 ], [ 0, 0, 0, 1, 0, 0, 0, 0 ], 
      [ 0, 0, 0, 0, 1, 0, 0, 0 ], [ 0, 0, 0, 0, 0, 1, 1, 0 ], 
      [ 0, 0, 0, 0, 0, 0, -1, 0 ], [ 0, 0, 0, 0, 0, 0, 1, 1 ] ], 
  [ [ 1, 0, 0, 0, 0, 0, 0, 0 ], [ 0, 1, 0, 0, 0, 0, 0, 0 ], 
      [ 0, 0, 1, 0, 0, 0, 0, 0 ], [ 0, 0, 0, 1, 0, 0, 0, 0 ], 
      [ 0, 0, 0, 0, 1, 0, 0, 0 ], [ 0, 0, 0, 0, 0, 1, 0, 0 ], 
      [ 0, 0, 0, 0, 0, 0, 1, 1 ], [ 0, 0, 0, 0, 0, 0, 0, -1 ] ] ];

	gens := List(E8gens, m -> m{[1..n]}{[1..n]});
	H := Group(gens * One(GF(3)));

	if n = 6 then
		Assert(0, 51840 = Size(H));
	elif n = 7 then
		Assert(0, 2903040 = Size(H));
	elif n = 8 then
		Assert(0, 696729600 = Size(H));
	fi;
	
	G := SemidirectProduct(H, GF(3)^n);
	d1 := G.1;

	Print("Computing transvections\n");
	D := Elements(ConjugacyClass(G, d1));;
	Print("|D| = ", Size(D), "\n");

	# For E_6:
	# |D| = 108
	# dim(A/V) = 78
	# dim(V) = 30
	# dim(L) = 78

	# For E_7:
	# |D| = 189
	# dim(A/V) = 132
	# dim(V) = 57
	# dim(L) = 133

	# For E_8:
	# |D| = 360
	# dim(A/V) = 248
	# dim(V) = 112
	# dim(L) = 248

elif n in [ 1001..1020 ] then

	Print("Sporadic example ", n, "\n");

	F := FreeGroup("a","b","c","d","e","f","g","h","r");
	a:=F.1; b:=F.2; c:=F.3; d:=F.4; e:=F.5; f:=F.6; g:=F.7; h:=F.8; r:=F.9;
	x := r;
	#AssignGeneratorVariables(f);
	#ASS_GVAR(s, g);
	if n in [ 1001..1002 ] then
		edges := [ [a,b], [b,c], [b,d], [b,x], [c,d], [c,x], [d,e], [d,x], [e,f] ];
		rels := [ (b^c*d)^3, (b^c*x)^3, (b^d*x)^3, (c^d*x)^3,
					(b^c*d^x)^3, (b^d*c^x)^3, (b^x*c^d)^3,
					(a*c^((b*d*x)^2))^2, (e*c^((b*d*x)^2))^2
					];
		Hgens := [ b, e, f, b^(a*d), (b*c*d)^2, (b*c*x)^2, (b*d*x)^2 ];
		if n = 1001 then
			# Soicher-Hall 8.4 (for |I|=1)  /  PR.13:    (3^5 . 3^5) : ^+Omega^-( 5, 3 )
			# |D| = 405 = 9*45, dim L = 0, dim A/V = 374, dim V = 31
			# computing from scratch yields  dim L = TODO
			forbidden_gens := [f, g, h];
		elif n = 1002 then
			# Soicher-Hall 8.5 (for |I|=1)  /  PR.14:    (3^6 . 3^6) : (3 . ^+Omega^-( 6, 3 ))
			# |D| = 1134 = 9*126, dim L = 0, dim A/V = 1042, dim V = 92
			# computing from scratch yields  dim L = TODO
			forbidden_gens := [g, h];
		fi;
	elif n in [ 1003..1005 ] then
		edges := [ [a,b], [b,c], [b,d], [c,d], [d,e], [d,h], [a,r], [e,f], [f,g] ];
		rels := [ (b^c*d)^3, (e^((b*c*d)^2)*h)^2 ];
		Hgens := [ b, e, h, b^(a*d), (b*c*d)^2, r, f, g ];
		if n = 1003 then
			# Soicher-Hall 8.6 (for |I|=1):    Omega^+( +1, 6, 3 )
			# |D| = 117, dim L = 52, dim A/V = 26, dim V = 91
			forbidden_gens := [r, f, g];
		elif n = 1004 then
			# Soicher-Hall 8.7 (for |I|=1)  /  PR.15:    3^7 . Omega^-( -1, 7, 3 )
			# |D| = 1134 = 3*378, dim L = 0, dim A/V = 860, dim V = 274
			forbidden_gens := [f, g];
		elif n = 1005 then
			# Soicher-Hall 8.8 (for |I|=1)  /  PR.16:    3^8 . Omega^-( -1, 8, 3 )
			# |D| = 3321 = 3*1107, dim L = 0, dim A/V = 2500, dim V = 821
			forbidden_gens := [r];
		fi;
	elif n in [ 1006..1010 ] then
		edges := [ [a,b], [a,c], [a,d], [b,c], [c,d], [d,e], [e,f], [e,h], [f,g] ];
		rels := [ (a^c*b)^3, (a^c*d)^3, (a^b*c^d)^3 ];
		Hgens := [ d, b, f, g, h, d^(e*a), (a*c*d)^2, (a^b*c*d)^2 ];
		if n <> 1006 then
			Add(rels, h^-1*((a*c*b)^-2*(a*c*d)^2)^2*(a*b*c*d*e)^15);
		fi;
		if n = 1006 then
			# Soicher-Hall 8.9: 2 x SU(5,2)
			# |D| = 165, dim L = 45, dim A/V = 24, dim V = 141
			forbidden_gens := [f, g, h, r];
		elif n = 1007 then
			# Soicher-Hall 8.10: 2 x (2^14 . SU(7,2)
			# |D| = 10836 = 4*2709, dim L <= 133, dim A/V = 48, dim(V) = 10788
			Add(rels, (a*(b*c*d*e*f*h)^5*(b*c*d*e*f*g*h)^9)^2);
			forbidden_gens := [r];
		elif n = 1008 then
			# Soicher-Hall 8.11: 2 x (2^3 . (2^10 . 2^10) : SU(5,2))
			# |D| = 2640 = 16*165, dim L = 88, dim A/V = 24, dim V = 2616
			forbidden_gens := [g, r];
		elif n = 1009 then
			# Soicher-Hall 8.12(1) (for |I|=2) /  PR.11: 3^3 . (3^10 . 3^10) : (2 x SU(5,2))
			# |D| = 13365 = 81 * 165, dim L <= 45, dim A/V = 13224, dim V = 141
			Add(edges, [f,h]);
			Add(rels, (e^f*h)^3);
			forbidden_gens := [g, r];
		elif n = 1010 then
			# Soicher-Hall 8.12(2) (for |I|=1) /  PR.11: 3^(1+10) : (2 x SU(5,2))
			# |D| = 1485 = 9 * 165, dim L = 0, dim A/V = 1344, dim V = 141
			Add(edges, [f,h]);
			Add(rels, (e^f*h)^3);
			Add(rels, Comm(e^((a*c*d)^2), (e*f*h)^2));
			forbidden_gens := [g, r];
		fi;
	elif n in [ 1011..1012 ] then
		edges := [ [a,b], [b,d], [d,c], [c,b], [b,e], [e,c], [c,a] ];
		rels := [ (b^a*c)^3, (b^d*c)^3, (a^b*c^d)^3,
					(b^c*e)^3, (b^(d*c)*e)^3, (b^(a*c)*e)^3,
					(d^((a*b*c)^-2)*e)^3, (d^((a*b*c)^2)*e)^2
				];
		Hgens := [ e, a, d, (e*b*c)^2, (e*b*c^a)^2, (e*b*c^d)^2,
					c^(a*d*e*c), b^(a*d*e*b) ];
		forbidden_gens := [f, g, r];
		if n = 1011 then
			# Soicher-Hall A.24 (for |I|=2) /  PR.12: 3^3.(3^8+3^8):2^(1+6):SU(3,2)'
			# |D| = 2916, dim L = 0, dim A/V = 2888, dim V = 28
		elif n = 1012 then
			# Soicher-Hall A.25 (for |I|=1) /  PR.12: 3^(1+8):2^(1+6):SU(3,2)'
			# |D| = 324, dim L = 0, dim A/V = 296, dim V = 28
			Add(rels, Comm(e^((a*b*c)^2), (b*c*d)^2));
		fi;
	fi;

	# Soicher-Hall XXX  /  PR.9:
	# Soicher-Hall XXX  /  PR.10:


	edges := Filtered(edges, e->IsEmpty(Intersection(forbidden_gens, e)));

	CoxeterQuotient := function( F, edges, rels, Hgens )
		local gens, i, j, Gfp, Hfp;
		edges := List(edges, Set);
		rels := ShallowCopy( rels );
		Append(rels, forbidden_gens);
		gens := GeneratorsOfGroup(F);
		for i in [1..Size(gens)] do
			Add(rels, gens[i]^2);
			for j in [i+1..Size(gens)] do
				if [gens[i],gens[j]] in edges then
					Add(rels, (gens[i]*gens[j])^3);
				else
					Add(rels, (gens[i]*gens[j])^2);
				fi;
			od;
		od;

		# Compute the group G and a permutation presentation
		# on the subgroup H, the centralizer of a 3-transposition
		Gfp := F / rels;
		Hfp := Subgroup(Gfp, List(Hgens, w -> ElementOfFpGroup( FamilyObj(Gfp.1), w)));
		return Image( FactorCosetAction( Gfp, Hfp ) );;
	end;;

	G := CoxeterQuotient( F, edges, rels, Hgens );

	Print("Computing transvections\n");
	d1 := G.1;
	D := Elements(ConjugacyClass(G, d1));; 	# We know that a is an transvection
	Print("|D| = ", Size(D), "\n");
	
elif n = -73 then
	Print("Computing 3^7:SO(7,3)\n");
	Read("so73-ext.gap");
	G := H;

	Print("Computing transvections\n");
	
	# Compute all transvections = points
	D := Transvections(G);;
	Print("|D| = ", Size(D), "\n");
	d1 := D[1];
	
	# In SO(7,3), there are 378 transvections of plus type, and 351 of minus type.
	# In this group G; we have a transvection class of size 1134 = 3 * 378
	# Empirically, the classes of involutions have these sizes:
	# [ 26867295, 199017, 255879, 7164612, 10746918, 1134 ]
	# But only the last one is a 3-transposition class
	Assert(0, Size(D) = 1134);

elif n = -6 then
	#
	# Determine O_6^-(3) a subgroup of U_6(2)
	#

	Print("Computing O_6^-(3) a subgroup of U_6(2)\n");

	LoadPackage("atlasrep");
	U62 := AtlasGroup("U6(2)");
	a := U62.1;
	b := U62.2;
	
	# NOTE: there are three conjugacy classes!

	#G := Group(a, b^2*a*b^3);
	#G := Group(a, b*a*b*a*b^2*a*b^2*a*b^3);
	G := Group(a, b*a*b*a*b*a*b^3*a*b^3);
	d1 := a; 	# We know that a is an transvection

	Print("Computing transvections\n");
	D := Elements(ConjugacyClass(G, d1));;
	Print("|D| = ", Size(D), "\n");

elif n = -82 then
	Print("Computing O_8^+(2):S_3 a subgroup of Fi22\n");

	# Load rep of Fi22 (taken from the Atlas)
	LoadPackage("atlasrep");
	Fi22 := AtlasGroup("Fi22");
	a := Fi22.1;;
	b := Fi22.2;;
	# Subgroup O_8^+(2):S_3
	# Size 1045094400
	c := (b*a*b)^3*b^5;
	G := Group(a, c);
	d1 := a; 	# We know that a is an transvection

	Print("Computing transvections\n");
	D := Elements(ConjugacyClass(G, d1));;
	Print("|D| = ", Size(D), "\n");	# = 360
elif n = -83 then
	Print("Computing O_8^+(3):S_3 a subgroup of Fi23\n");

	# Load rep of Fi23 (taken from the Atlas)
	LoadPackage("atlasrep");
	Fi23 := AtlasGroup("Fi23");
	a := Fi23.1;;
	b := Fi23.2;;

	work := [];;
	output := [];;
	work[3] := a * b;;
	work[4] := work[3] * b;;
	work[5] := work[3]^3;;
	work[6] := work[4]^3;;
	work[4] := work[6] * work[5];;
	work[5] := work[3]^8;;
	work[3] := work[5] * work[4];;
	output[1] := work[4] * work[4];;
	output[2] := work[3] * work[5];;

	a := output[1];;
	b := output[2];;
	Assert(0, Order(a) = 2);

	# Subgroup O_8^+(3):S_3
	G := Group(a, b);
	Assert(0, Size(G) = 29713078886400);
	G := Image(SmallerDegreePermutationRepresentation(G));

	# Find a transvection by size of its conjugacy class
	Print("Find a transvection...\n");
	repeat
		d1 := PseudoRandom(G);
		if IsEvenInt(Order(d1)) then
			d1 := d1^(Order(d1)/2);
		fi;
		Display(Size(ConjugacyClass(G,d1)));
	until Order(d1) = 2 and Size(ConjugacyClass(G,d1))=3240;

	Print("Computing transvections\n");
	D := Elements(ConjugacyClass(G, d1));;
	Print("|D| = ", Size(D), "\n");	# = 3240
elif n = -22 then
	Print("Computing Fi22\n");

	LoadPackage("atlasrep");
	G := AtlasGroup("Fi22");

	a := G.1;;
	b := G.2;;
	d1 := a;;

	Print("Computing transvections\n");
	D := Elements(ConjugacyClass(G, d1));;
	Print("|D| = ", Size(D), "\n");	# = 3510
	Assert(0, Size(D) = 3510);

elif n = -23 then
	Print("Computing Fi23\n");

	# Load rep of Fi23
	LoadPackage("atlasrep");
	G := AtlasGroup("Fi23");
	
	a := G.1;;
	b := G.2;;
	d1:=(a*(b*(a*b)^3)^4*a*b^2)^15;;

	Print("Computing transvections\n");
	D := Elements(ConjugacyClass(G, d1));;
	Print("|D| = ", Size(D), "\n");	# = 31671
	Assert(0, Size(D) = 31671);
elif n = -24 then
	Print("Computing Fi24\n");

	# Load rep of Fi24
	LoadPackage("atlasrep");
	G := AtlasGroup("Fi24'.2");

	a := G.1;;
	b := G.2;;
	d1 := a;;

	Print("Computing transvections\n");
	D := Elements(ConjugacyClass(G, d1));;
	Print("|D| = ", Size(D), "\n");	# = 306936
	Assert(0, Size(D) = 306936);
elif n in [100..199] then
	n := n - 100;

	# TODO: Sp_2n(2)
	# - compute Lie algebra, verify dim 2^{2n}-1
	# - ideal generated by singular lines (3pts) -> give quotient of dim 2n+1
	# - ideal generated by hyperbolic lines -> quotient of dim 2n+1
	# - ideal generated by both: dim 2n
	Print("Computing Sp(", 2*n, ",2)\n");
	G := Sp(2*n, 2);

	Print("Computing transvections\n");
	D := Transvections(G);;
	Print("|D| = ", Size(D), "\n");
	d1 := D[1];
	Assert(0, Size(D) = 4^n-1);
elif n in [200..299] then
	# TODO: Orthogonal group over F_3
	Assert(0, sign in [-1,+1]);

	n := n - 200;
	if n mod 2 = 0 then
		Print("Computing O(", sign, ",", n, ",3)\n");
		G := GO(sign,n,3);

		# Compute a reflection
		d1 := IdentityMat(n,GF(3));
		d1{[1..2]}{[1..2]} := [[0,1],[1,0]]*One(GF(3));
		Assert(0, d1 in G);
		
		# for GO(+1,6,3), we get four classes of size 117, in each case dim(A/V) = 26; dim(L) = 52
		# for GO(-1,6,3), we get four classes of size 126, in each case dim(A/V) = 34; dim(L) = 56

	else
		if sign = +1 then
			Print("Computing + class in O(", n, ",3)\n");
		else
			Print("Computing - class in O(", n, ",3)\n");
		fi;
		G := GO(n,3);

		if sign = (-1)^((n-1)/2) then
			d1 := IdentityMat(n,GF(3));
			d1{[1..2]}{[1..2]} := [[0,1],[1,0]]*One(GF(3));
		else
			d1 := IdentityMat(n,GF(3));
			d1{[1..2]}{[1..2]} := -[[0,1],[1,0]]*One(GF(3));
			#d1[1][1]:= -One(GF(3));
		fi;
		#d1[3][3] := -One(GF(3));
		Assert(0, Order(d1) = 2);
		Assert(0, d1 in G);

		# for SO(3,3) we get dim 0 (from the size 3 space) and 2 (from the size 6 space)
		# for SO(5,3) we get dim 6 (from the size 36 space) and 14 (from the size 45 space)
		# 	Class 36:
		# 		dim(A/V) = 6; dim(L) = 36
		# 	Class 45:
		# 		dim(A/V) = 14; dim(L) = 30
	
		# for SO(7,3) we get dim 78 (from the size 351 space) and 104 (from the size 378 space)
		#                    the 351 space has an affine plane ideal of dim 273 (and 78 + 273 = 351)
		#                    the 378 space has an affine plane ideal of dim 378
	fi;

	Print("Computing transvections\n");
	D := Elements(ConjugacyClass(G, d1));;
	Print("|D| = ", Size(D), "\n");
	#Assert(0, Size(D) = XXXX);

else
	Error("Invalid parameter n = ", n, "\n");
fi;

if false then
	# Ensure G is generated by D
	tmp := Size(G);
#	G := Group(D);
	G := Group(List([1..20],i->PseudoRandom(D)));
	G := Group(SmallGeneratingSet(G));
	if Size(G) <> tmp then
		Display("!!!!!!!!!!!!!");
		Print("Reduced group generated by D has size ", Size(G), " instead of ", tmp, "\n");
		Display("!!!!!!!!!!!!!");
	fi;
fi;


# We need Gperm, a perm rep of G on D. In some cases, G is already given by
# such a perm rep; in others, we need to compute it.
if n in [-23, -24] then
	actHom:= IdentityMapping(G);
	Gperm := G;
else
	# Permutation rep of G on D
	Print("Computing permutation rep of G on D... ");
	actHom := ActionHomomorphism(G, D);
	Gperm := Image(actHom);
	d1 := Image(actHom,d1);;
fi;


if true then
	# Ensure G is generated by D
	tmp := Size(Gperm);
	Gperm := Group(Elements(ConjugacyClass(Gperm,d1)));
	Gperm := Group(SmallGeneratingSet(Gperm));
	if Size(Gperm) <> tmp then
		Display("!!!!!!!!!!!!!");
		Print("Reduced Gperm generated by D has size ", Size(Gperm), " instead of ", tmp, "\n");
		Display("!!!!!!!!!!!!!");
	fi;
fi;


# Ensure that the center of d1 *really* is 1
d1 := GetTransvectionWithNumber(Gperm, d1, 1);


Print("|G| = ", Size(G), "; |Gperm| = ", Size(Gperm), "\n");
